@@ -1,54 +1,71 @@
 """
+"""
 
-"""
-
+import inspect
 import os
 import sys
 from functools import wraps
-import inspect
 
-from .mixin import CallbackMix
+from lumo.base_classes.trickitems import NoneItem, AvgItem
+from lumo.proc.dist import is_main
+from lumo.utils.timer import format_second, Timer
 from .meter import AvgMeter
 from .meter import Meter
+from .mixin import CallbackMix
 from .params import Params
-from ..base_classes.trickitems import NoneItem, AvgItem
-from ..calculate.schedule import Scheduler, SchedulerList
-from ..utils.timing import format_second
-from typing import TYPE_CHECKING
+from .trainer import Trainer, TrainerResult
 
-func_map = {
+_func_map = {
     'evaluate': 'eval',
     'evaluate_step': 'eval_step'
 }
 
-from .trainer import Trainer, TrainerResult
-
-
-# if TYPE_CHECKING:
-
 
 def map_func_name(name):
-    return func_map.get(name, name)
+    return _func_map.get(name, name)
 
 
-class BaseCallback():
+class BaseCallback:
     """
-    base callback class
+    Abstract base class used to build new callbacks.
 
-    only have two methods `on_begin()` and `on_end()`.
+    Callbacks are created to hook into the various stages of lumo's Trainer training, inference or data preparing.
+    You can hook any subclasses of `BaseCallback` in `lumo.kit.trainer.Trainer` class.
+
+    To create a custom callback class, subclass `lumo.kit.callbacks.BaseCallback`, and define the callback methods
+    you want to do something before or after that.
+
+    The callback method must be named like on_xxx_begin()/on_xxx_end(), where `xxx` is a function in trainer.
+
+
+    TrainCallback, SaveLoadCallback and InitialCallback define the most methods in Trainer, you can simply subclass these
+    callback class to custom your callback.
+
+    Notes:
+        There is no need for you to call the on_xxx function in your function's logic.
+
+        The Trainer has some magic feature... not like keras or other framework with trainer/callback logic that
+        you may need to manully call the callback function `on_xxx...()` in you overrided or newly defined method in Trainer,
+        If you define a function `foo()` in your trainer, the callback function on_foo_begin() and on_foo_end() will be found
+        automatically and be called, There is no need for you to call them in your function's logic.
+
+
+    Examples:
+        {{examples/docs/basecallback.py}}
+
+        {{examples/docs/basecallback2.py}}
 
-    for simpler using, see TrainCallback.
     """
-    priority = 0  # type:int # All callbacks in lumo will have priority in range 0-100
-    only_single_gpu = False  # only hooked in single gpu mode
-    only_main_process = False  # whether can be hooked in children process( local_rank > 0)
+    priority = 0  # type: int # All callbacks in lumo will have priority in range 0-100
+    only_single_gpu = False  # Callback only works when single gpu mode
+    only_main_process = False  # Callback only works when in main process
 
     def __new__(cls, *_, **__):
         self = super().__new__(cls)
         self._hooked = None
 
         def ecp_wrap(func):
-            """同一个异常第一次调用的时候运行"""
+            """Wrapper for cache exceptions"""
 
             @wraps(func)
             def on_exception(hooked: Trainer, tfunc, params: Params, e: BaseException, *args, **kwargs):
@@ -131,7 +148,14 @@ class BaseCallback():
 
 class TrainCallback(BaseCallback):
     """
-    实现了一般训练过程中的函数函数回调，主要将 on_begin() / on_end() 方法分发到各具体的回调方法中
+    Abstract base class used to build new callbacks. Defined some methods used in train/eval/test loop.
+
+    Subclass TrainCallback if you want to control or add behavior before or after
+        - train/train_epoch/train_step
+        - eval/eval_epoch/eval_step
+        - test/test_epoch/test_step
+        - predict
+        - inference
     """
 
     def on_train_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
@@ -190,6 +214,14 @@ class TrainCallback(BaseCallback):
 
 
 class SaveLoadCallback(BaseCallback):
+    """
+    Abstract base class used to build new callbacks. Defined some methods used in save/load.
+
+    Subclass SaveLoadCallback if you want to control or add behavior before or after
+        - save_xxx
+        - load_xxx
+    """
+
     def on_save_keypoint_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
         pass
 
@@ -216,6 +248,13 @@ class SaveLoadCallback(BaseCallback):
 
 
 class InitialCallback(BaseCallback):
+    """
+    Abstract base class used to build new callbacks. Defined some methods used in save/load.
+
+    Subclass InitialCallback if you want to control or add behavior before or after
+        - initialization of optims/models/dataloader
+    """
+
     def on_ioptims_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
         pass
 
@@ -237,10 +276,16 @@ class InitialCallback(BaseCallback):
 
 class EvalCallback(TrainCallback):
     """
+    Callback for evaluation.
+
+    Args:
+        eval_per_epoch:
+        test_per_epoch:
     """
     only_main_process = True
 
     def __init__(self, eval_per_epoch=1, test_per_epoch=10):
+
         self.eval_in_per_epoch = eval_per_epoch
         self.test_in_per_epoch = test_per_epoch
 
@@ -248,12 +293,20 @@ class EvalCallback(TrainCallback):
         self._last_eval = -1
         self._last_test = -1
 
+    @property
+    def has_eval(self):
+        return self.eval_in_per_epoch is not None and self.eval_in_per_epoch > 0
+
+    @property
+    def has_test(self):
+        return self.test_in_per_epoch is not None and self.test_in_per_epoch > 0
+
     def _test_or_eval(self, params: Params, trainer: Trainer):
-        if self.eval_in_per_epoch is not None and self.eval_in_per_epoch > 0:
+        if self.has_eval:
             if params.eidx % self.eval_in_per_epoch == self.eval_in_per_epoch - 1:
                 self._last_eval = params.eidx
                 trainer.evaluate()
-        if self.test_in_per_epoch is not None and self.test_in_per_epoch > 0:
+        if self.has_test:
             if params.eidx % self.test_in_per_epoch == self.test_in_per_epoch - 1:
                 self._last_test = params.eidx
                 trainer.test()
@@ -262,9 +315,9 @@ class EvalCallback(TrainCallback):
         self._test_or_eval(params, trainer)
 
     def on_train_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
-        if self._last_eval != params.eidx:
+        if self._last_eval != params.eidx and self.has_eval:
             trainer.evaluate()
-        if self._last_test != params.eidx:
+        if self._last_test != params.eidx and self.has_test:
             trainer.test()
 
     def __repr__(self):
@@ -273,18 +326,17 @@ class EvalCallback(TrainCallback):
 
 class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
     """
-    用于日志输出的回调，当 BaseTrainer 在 epoch / batch 等级别的训练结束、异常发生等过程后，Logger 会对这些事件，
-    或方法返回的结果进行输出。
-
-    一般情况下 Logger 支持所有类型输出，但如果使用 Meter 类进行包装，会有更好的输出形式
+    Callback to log info produced during whole Trainer lifecycle.
     """
-    only_main_process = True
+    # only_main_process = True
     priority = 100
 
     def __init__(self, avg=True, step_frequence=3, breakline_in=1000):
         self.avg = avg
         self.step_frequence = step_frequence
         self.breakline = breakline_in
+        self.history_loader = set()
+        self.is_main = is_main()
 
     @property
     def meter(self):
@@ -300,9 +352,13 @@ class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
         self._meter = meter
 
     def _need_log(self, step):
+        if not self.is_main:
+            return False
         return self.step_frequence > 0 and step % self.step_frequence == 0
 
     def _need_breakline(self, step):
+        if not self.is_main:
+            return False
         return self.breakline > 0 and step % self.breakline == 0
 
     def on_hooked(self, source: Trainer, params: Params):
@@ -316,10 +372,8 @@ class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
         self._meter = None
 
     def on_train_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
-        from ..utils.timing import TimeIt
-
         self.start = params.eidx
-        self.traintime = TimeIt()
+        self.traintime = Timer()
         self.traintime.start()
         trainer.logger.info('[[Train Begin]]')
         super().on_train_begin(trainer, func, params, *args, **kwargs)
@@ -332,9 +386,8 @@ class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
         trainer.logger.info(f"[[Train End in {format_second(self.traintime['use'])}]]", meter)
 
     def on_train_epoch_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
-        from ..utils.timing import TimeIt
         self.reset_meter()
-        self.epochtime = TimeIt()
+        self.epochtime = Timer()
         self.epochtime.start()
         trainer.logger.info("{}/{}".format(params.eidx, params.epoch))
 
@@ -353,10 +406,11 @@ class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
         tm.epoch = format_second(self.epochtime["use"])
         tm.avg = format_second(avg)
         tm.last = format_second(last)
+        # trainer.logger.newline()
         trainer.logger.info(tm)
 
     def on_train_step_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
-        meter = trainer._wrap_result(meter)
+        meter = Meter.wrap_result(meter)
         self.meter.update(meter)
         meter = self.meter
         if self._need_breakline(params.idx):
@@ -369,36 +423,37 @@ class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
         source.logger.error("{} raised".format(e.__class__.__name__))
 
     def on_test_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
+        trainer.logger.newline()
         self.reset_meter()
-        trainer.logger.info("[[Test Begin]]")
 
     def on_test_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
         meter = result.meter
         if meter is None:
             meter = ""
-        trainer.logger.info("[[Test End]]", meter)
+        trainer.logger.info("[[Test]]", meter)
+        self.reset_meter()
 
     def on_test_step_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
-        meter = trainer._wrap_result(meter)
+        meter = Meter.wrap_result(meter)
         self.meter.update(meter)
         meter = self.meter
-        trainer.logger.inline("{}/{}".format(params.idx + 1, len(trainer.test_dataloader)), meter, fix=1)
+        trainer.logger.inline("[[Test]]", "{}/{}".format(params.idx + 1, len(trainer.test_dataloader)), meter, fix=1)
 
     def on_eval_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
+        trainer.logger.newline()
         self.reset_meter()
-        trainer.logger.info("[[Eval Begin]]")
 
     def on_eval_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
         meter = result.meter
         if meter is None:
             meter = ""
-        trainer.logger.info("[[Eval End]]", meter)
+        trainer.logger.info("[[Eval]]", meter)
 
     def on_eval_step_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
-        meter = trainer._wrap_result(meter)
+        meter = Meter.wrap_result(meter)
         self.meter.update(meter)
         meter = self.meter
-        trainer.logger.inline("{}/{}".format(params.idx + 1, len(trainer.val_dataloader)), meter, fix=1)
+        trainer.logger.inline("[[Eval]]", "{}/{}".format(params.idx + 1, len(trainer.val_dataloader)), meter, fix=1)
 
     def on_save_checkpoint_end(self, trainer: Trainer, func, params: Params, result: str, *args, **kwargs):
         trainer.logger.info(f'Saved checkpoint in {result}')
@@ -425,6 +480,12 @@ class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
         trainer.logger.info('[[Model initialized]]')
 
     def on_prepare_dataloader_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
+        # print(meter)
+        loader_id = id(meter)
+        if loader_id in self.history_loader:
+            return
+
+        self.history_loader.add(loader_id)
         res = self.getfuncargs(func, *args, **kwargs)
         stage = res['stage'].name
         loader_ = trainer.datamodule[stage]
@@ -436,8 +497,7 @@ class LoggerCallback(TrainCallback, InitialCallback, SaveLoadCallback):
 
 class MeterCheckpoint(TrainCallback):
     """
-    用于检视训练过程中模型的某个指标，并根据其提升进行 checkpoint 类型的保存
-    该类参考了 Keras 中相应的实现。
+    Callback to save the model checkpoints by a monitored metric get it's best value.
     """
     only_main_process = True
 
@@ -497,7 +557,7 @@ class EpochCheckpoint(TrainCallback):
     def on_train_epoch_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
         meter = result.meter
         if params.eidx % self.per_epoch == 0 and params.eidx > 0:
-            trainer.save_keypoint(meta_info=trainer._wrap_result(meter))
+            trainer.save_keypoint(meta_info=Meter.wrap_result(meter))
 
     def __repr__(self) -> str:
         return self._repr_by_val("per_epoch")
@@ -512,10 +572,13 @@ class GlobalStepCheckpoint(TrainCallback):
     def on_train_step_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
         super().on_train_step_end(trainer, func, params, meter, *args, **kwargs)
         if params.global_step % self.per == 0 and params.global_step > 0:
-            trainer.save_checkpoint(meta_info=trainer._wrap_result(meter))
+            trainer.save_checkpoint(meta_info=Meter.wrap_result(meter))
 
 
 class KeyErrorSave(TrainCallback):
+    """
+    Callback to save checkpoints when you interrupt the program.
+    """
     only_main_process = True
     only_single_gpu = True
     priority = -1
@@ -536,45 +599,55 @@ class KeyErrorSave(TrainCallback):
                 return True
 
 
-# class BoardRecord(TrainCallback):
-#     """
-#     自动记录训练过程中的所有变量到 tensorboard 中（epoch 级）
-#     """
-#     only_main_process = True
-#     priority = 100
-#
-#     def __init__(self) -> None:
-#         super().__init__()
-#
-#     def on_hooked(self, source: Trainer, params: Params):
-#         self.start = 0
-#
-#     def _key_name(self, mode, key):
-#         return "{}_{}_".format(key, mode)
-#
-#     def on_test_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
-#         meter = result.meter
-#         if isinstance(meter, Meter):
-#             for k, v in meter.numeral_items():
-#                 trainer.writer.add_scalar(self._key_name("test", k), v, params.eidx)
-#
-#     def on_train_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
-#         self.start = params.eidx
-#
-#     def on_eval_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
-#         meter = result.meter
-#         if isinstance(meter, Meter):
-#             for k, v in meter.numeral_items():
-#                 trainer.writer.add_scalar(self._key_name("eval", k), v, params.eidx)
-#
-#     def on_train_epoch_end(self, trainer: Trainer, func, params: Params, meter: AvgMeter, *args, **kwargs):
-#         if isinstance(meter, Meter):
-#             for k, v in meter.numeral_items():
-#                 trainer.writer.add_scalar(self._key_name("train", k), v, params.eidx)
+class ScalarRecorder(TrainCallback):
+    """
+    This callback logs scalars yield in train/test/eval by TensorBoard.
+    """
+    only_main_process = True
+    priority = 100
+
+    def __init__(self, writer=None) -> None:
+        super().__init__()
+        self.writer = writer
+
+    def _key_name(self, mode, key):
+        return "{}_{}_".format(key, mode)
+
+    def write_scalar(self, meter, stage: str, global_step, writer):
+        if isinstance(meter, (Meter, AvgMeter)):
+            for k, v in meter.scalar_items():
+                writer.add_scalar(self._key_name(stage, k), v, global_step)
+
+    def on_test_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
+        meter = result.meter
+        writer = self.writer
+        if writer is None:
+            writer = trainer.writer
+        self.write_scalar(meter, 'test', trainer.global_step, writer)
+
+    def on_eval_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
+        meter = result.meter
+        writer = self.writer
+        if writer is None:
+            writer = trainer.writer
+        self.write_scalar(meter, 'eval', trainer.global_step, writer)
+
+    def on_train_epoch_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
+        meter = result.meter
+        writer = self.writer
+        if writer is None:
+            writer = trainer.writer
+        self.write_scalar(meter, 'train', trainer.global_step, writer)
 
 
 class EMAUpdate(TrainCallback):
-    only_main_process = True
+    """
+    Callback to update EMA model every train step.
+
+    Variable in Trainer instance is identified as EMA model when
+     - is instance of torch.nn.Module
+     - name is started with 'ema'
+    """
 
     def on_train_step_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
         super().on_train_step_end(trainer, func, params, meter, *args, **kwargs)
@@ -583,60 +656,14 @@ class EMAUpdate(TrainCallback):
                 v.step()
 
 
-# class LRSchedule(TrainCallback):
-#     def __init__(self, schedule: Schedule = None, apply=True, use_eidx=True):
-#         self.schedule = schedule
-#         self.apply = apply
-#         self.use_eidx = use_eidx
-#
-#     def on_hooked(self, source: Trainer, params: Params):
-#         super().on_hooked(source, params)
-#         if self.schedule is None:
-#             if 'lr_sche' not in params:
-#                 source.logger.warn(f'lr_sche not exists in params and be assigned, {self} will be unhooked after.')
-#                 self.unhook()
-#             else:
-#                 self.schedule = params.lr_sche
-#
-#     def on_train_epoch_begin(self, trainer: Trainer, func, params: Params, *args, **kwargs):
-#         for k, v in trainer.optim_dict.items():
-#             if self.use_eidx:
-#                 step = params.eidx
-#             else:
-#                 step = params.global_step
-#
-#             if self.apply:
-#                 new_lr = self.schedule.apply(v, step)
-#                 trainer.logger.info('{}.lr = {}'.format(k, new_lr))
-#             else:
-#                 ratio = self.schedule.scale(v, step)
-#                 trainer.logger.info('lr scale ratio = {}'.format(k, ratio))
-
-
-# class ReportSche(TrainCallback):
-#     """
-#     log `schedule` in every epoch end
-#     `schedule` means `Schedule` in Params and have `sche` in the name, which will have different value in every epoch
-#     """
-#     only_main_process = True
-#     priority = 100
-#
-#     def on_hooked(self, source: Trainer, params: Params):
-#         self.sche_lis = []
-#         for k, v in params.items():  # type:str, Any
-#             if isinstance(v, (Schedule, ScheduleList)) and 'sche' in k.lower():
-#                 self.sche_lis.append((k, v))
-#
-#     def on_train_epoch_end(self, trainer: Trainer, func, params: Params, result: TrainerResult, *args, **kwargs):
-#         super().on_train_epoch_end(trainer, func, params, result, *args, **kwargs)
-#         meter = result.meter
-#         m = Meter()
-#         for k, v in self.sche_lis:
-#             m[k] = v(params.eidx)
-#         trainer.logger.info(m)
-
-
 class AutoLoadModel(InitialCallback):
+    """
+    Callback to automatically load pretrained model.
+
+    The load function will be executed if and only if
+     - both the params `pretrain` and `pretrain_path` are defined in Params
+     - `pretrain` is True and `pretrain_path` is not None.
+    """
 
     def on_imodels_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
         super().on_imodels_end(trainer, func, params, meter, *args, **kwargs)
@@ -647,6 +674,9 @@ class AutoLoadModel(InitialCallback):
 
 
 class EvalFirst(AutoLoadModel):
+    """
+    Callback to evaluation before the first train step. Aften used to debug or evaluate some dataset by restored model.
+    """
 
     def __init__(self, datamodule=None):
         super().__init__()
@@ -659,3 +689,21 @@ class EvalFirst(AutoLoadModel):
                 trainer.evaluate(self.datamodule)
             else:
                 trainer.evaluate()
+
+
+class RemoteMonitor(TrainCallback, InitialCallback, SaveLoadCallback):
+    pass  # TODO
+
+
+class StopByCode(TrainCallback):
+    def __init__(self, step=100):
+        self.step = step
+
+    def on_train_step_end(self, trainer: Trainer, func, params: Params, meter: Meter, *args, **kwargs):
+        super().on_train_step_end(trainer, func, params, meter, *args, **kwargs)
+        if trainer.global_step % self.step == 0:
+            if os.path.exists(trainer.exp.test_file('.stop')):
+                trainer.exp.add_tag('lumo.early_stop')
+                trainer.logger.info('Early stop this test manully.')
+                trainer.stop_train_epoch()
+                trainer.stop_train()
