@@ -94,7 +94,7 @@ class Merge(type):
     def __new__(cls, name, bases, attrs: dict, **kwds):
         for base in bases:
             for key, value in base.__dict__.items():  # type:(str,Any)
-                if not key.startswith("_"):
+                if key.endswith("__"):
                     continue
                 if isinstance(value, set):
                     v = attrs.setdefault(key, set())
